package com.aliyun.sts.impl;

import com.alibaba.dcm.DnsCacheManipulator;
import com.aliyun.sts.FederationToken;
import com.aliyun.sts.StsException;
import com.aliyun.sts.utils.Utils;
import com.aliyuncs.http.ProtocolType;
import org.junit.Before;
import org.junit.Test;

import java.util.Date;

import static com.aliyun.sts.Configs.ACCESS_KEY_ID;
import static com.aliyun.sts.Configs.ACCESS_KEY_SECRET;
import static com.aliyun.sts.Configs.GRANTEE;
import static com.aliyun.sts.Configs.POLICY_ALLOW_ALL;
import static com.aliyun.sts.Configs.POLICY_ALLOW_A_LOT;
import static com.aliyun.sts.Configs.SECONDS_OF_45Min;
import static org.hamcrest.CoreMatchers.endsWith;
import static org.hamcrest.core.StringStartsWith.startsWith;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertThat;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

public class StsServiceTest {
    private StsServiceImpl stsService = new StsServiceImpl();

    @Before
    public void setUp() throws Exception {
        DnsCacheManipulator.loadDnsCacheConfig();

        System.out.println(DnsCacheManipulator.getWholeDnsCache());
        Utils.disableSslVerification();
    }

    @Test
    public void test_getFederationToken() throws Exception {
        final long expireSeconds = SECONDS_OF_45Min;
        final String policy = POLICY_ALLOW_ALL;

        final FederationToken federationToken = stsService.getFederationToken(
                ACCESS_KEY_ID, ACCESS_KEY_SECRET, GRANTEE,
                policy, expireSeconds);
        System.out.printf("%s\n%s\n", policy, expireSeconds);
        assertNotNull(federationToken);
        System.out.println("getFederationToken: " + federationToken);

        assertThat(federationToken.getFederatedUser(), endsWith("federated-user/" + GRANTEE));
        assertThat(federationToken.getFederatedUser(), startsWith("acs:sts::"));

        assertTrue(federationToken.getRequestId().trim().length() > 0);

        assertTrue(federationToken.getAccessKeyId().trim().length() > 0);
        assertTrue(federationToken.getAccessKeySecret().trim().length() > 0);
        assertTrue(federationToken.getSecurityToken().trim().length() > 0);

        assertDateEqualsWithinDelta(new Date(System.currentTimeMillis() + expireSeconds * 1000),
                federationToken.getExpiration(),
                10);
    }

    @Test
    public void test_httpProtocolShouldFail() throws Exception {
        try {
            stsService.getFederationToken0(
                   ACCESS_KEY_ID, ACCESS_KEY_SECRET, GRANTEE,
                   POLICY_ALLOW_A_LOT, SECONDS_OF_45Min, ProtocolType.HTTP);
            fail();
        } catch (StsException expected) {
            assertEquals("InvalidProtocol.NeedSsl", expected.getErrorCode());
        }
    }

    static void assertDateEqualsWithinDelta(Date expected, Date actual, long deltaSeconds) {
        final long actualSeconds = actual.getTime() / 1000;
        final long expectedSeconds = expected.getTime() / 1000;
        final long actualDeltaSeconds = Math.abs(actualSeconds - expectedSeconds);

        String message = String.format("expected: %s, actual: %s, actual delta %ss > expected delta %ss",
                expected, actual, actualDeltaSeconds, deltaSeconds);
        assertTrue(message, expectedSeconds - deltaSeconds <= actualSeconds && actualSeconds <= expectedSeconds + deltaSeconds);
    }

    static void assertEqualsWithinDelta(String message, long expected, long actual, long delta) {
        delta = Math.abs(delta);
        if (message == null) {
            message = String.format("expected: %s, actual: %s, actual delta %s > expected delta %s",
                    expected, actual, Math.abs(actual - expected), delta);
        }

        assertTrue(message, expected - delta <= actual && actual <= expected + delta);
    }

    @Test
    public void test_errorMessage() throws Exception {
        try {
            stsService.getFederationToken(
                    ACCESS_KEY_ID, ACCESS_KEY_SECRET, GRANTEE,
                    POLICY_ALLOW_A_LOT, 3600 * 365);
            fail();
        } catch (StsException expected) {
            assertEquals("InvalidParameter.DurationSeconds", expected.getErrorCode());
        }
    }
}
