package com.aliyun.sts;

import com.alibaba.dcm.DnsCacheManipulator;
import com.aliyun.oss.OSSClient;
import com.aliyun.oss.OSSException;
import com.aliyun.oss.model.Bucket;
import com.aliyun.oss.model.OSSObject;
import com.aliyun.oss.model.ObjectMetadata;
import com.aliyun.sts.impl.StsServiceImpl;
import com.aliyun.sts.utils.Utils;
import org.apache.commons.io.IOUtils;
import org.junit.BeforeClass;
import org.junit.Test;

import java.io.ByteArrayInputStream;
import java.util.List;

import static com.aliyun.sts.Configs.ACCESS_KEY_ID;
import static com.aliyun.sts.Configs.ACCESS_KEY_SECRET;
import static com.aliyun.sts.Configs.BUCKET_NAME;
import static com.aliyun.sts.Configs.ENDPOINT;
import static com.aliyun.sts.Configs.GRANTEE;
import static com.aliyun.sts.Configs.OSS_FILE_PATH;
import static com.aliyun.sts.Configs.POLICY_ALLOW_A_LOT;
import static com.aliyun.sts.Configs.POLICY_NO_GETOBJECT;
import static com.aliyun.sts.Configs.SECONDS_OF_45Min;
import static org.hamcrest.CoreMatchers.startsWith;
import static org.junit.Assert.assertArrayEquals;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertThat;
import static org.junit.Assert.fail;
/**
 * @author ding.lid
 */
public class StsOssIntegrationTest {
    private static StsService stsService = new StsServiceImpl();
    private OSSClient ossClient;
    private FederationToken federationToken;

    
    
    @BeforeClass
    public static void beforeClass() throws Exception {
        DnsCacheManipulator.loadDnsCacheConfig();
        Utils.disableSslVerification();
    }

    public void setFederationTokenAndOssClient(String policy) {
        federationToken = stsService.getFederationToken(
                ACCESS_KEY_ID, ACCESS_KEY_SECRET, GRANTEE,
                policy, SECONDS_OF_45Min);
        System.out.println("getFederationToken: " + federationToken);

        ossClient = new OSSClient(ENDPOINT,
                federationToken.getAccessKeyId(), federationToken.getAccessKeySecret(),
                federationToken.getSecurityToken());
    }

    // FIXME 目前 listBuckets操作 不支持在Policy中配置！ 
    // @Test
    public void test_listBuckets() throws Exception {
        setFederationTokenAndOssClient(POLICY_ALLOW_A_LOT);

        final List<Bucket> buckets = ossClient.listBuckets();
        System.out.println("List Bucket:");
        for (Bucket bucket : buckets) {
            final String name = bucket.getName();
            System.out.println("\t" + name);
        }
    }

    @Test
    public void test_putObject() throws Exception {
        setFederationTokenAndOssClient(POLICY_ALLOW_A_LOT);

        String text = "中华人民共和国。Hello!";
        byte[] content = text.getBytes("UTF-8");

        ObjectMetadata meta = new ObjectMetadata();
        meta.setContentLength(content.length);

        ossClient.putObject(BUCKET_NAME, OSS_FILE_PATH, new ByteArrayInputStream(content), meta);

        OSSObject object = ossClient.getObject(BUCKET_NAME, OSS_FILE_PATH);
        byte[] bytes = IOUtils.toByteArray(object.getObjectContent());

        assertArrayEquals(content, bytes);
        System.out.println("putObject: Success");
    }

    @Test
    public void test_getObject() throws Exception {
        setFederationTokenAndOssClient(POLICY_ALLOW_A_LOT);

        String text = "中华人民共和国。Hello!";
        byte[] content = text.getBytes("UTF-8");

        OSSObject object = ossClient.getObject(BUCKET_NAME, OSS_FILE_PATH);
        byte[] bytes = IOUtils.toByteArray(object.getObjectContent());

        assertArrayEquals(content, bytes);
        System.out.println("getObject: Success");
    }

    @Test
    public void test_getObject_deny() throws Exception {
        setFederationTokenAndOssClient(POLICY_NO_GETOBJECT);

        try {
            ossClient.getObject(BUCKET_NAME, OSS_FILE_PATH);
            fail();
        } catch (OSSException expected) {
            assertEquals("AccessDenied", expected.getErrorCode());
            assertThat(expected.getMessage(), startsWith("Access denied by authorizer's policy"));
        }

    }

    @Test
    public void test_deleteObject() throws Exception {
        setFederationTokenAndOssClient(POLICY_ALLOW_A_LOT);

        ossClient.deleteObject(BUCKET_NAME, OSS_FILE_PATH);
        System.out.println("deleteObject: Success");
    }
}
